﻿using System;
using System.Linq;
using SQLite;

namespace StationProbe
{
    internal class Database : IDisposable
    {
        private readonly SQLiteConnection _db;

        public Database(string file)
        {
            _db = new SQLiteConnection(file);
            _db.CreateTable<Image>();
            _db.CreateTable<Exam>();
            _db.CreateTable<BatchTask>();
        }

        public Database()
        {
            var drives = System.IO.DriveInfo.GetDrives();
            var largestDriver = drives.OrderByDescending(x => x.TotalFreeSpace).First();
            _db = new SQLiteConnection($"{largestDriver.Name}database.db");
            _db.CreateTable<Image>();
            _db.CreateTable<Exam>();
            _db.CreateTable<BatchTask>();
        }


        public void AddBatchTask(BatchTask batchTask)
        {
            _db.Insert(batchTask);
        }


        public void AddImage(Image image)
        {
            lock (this)
            {
                _db.Insert(image);
            }
        }

        public void AddExam(Exam exam)
        {
            _db.Insert(exam);
        }

        public BatchTask[] GetBatchTasks()
        {
            return _db.Table<BatchTask>().ToArray();
        }

        public BatchTask? GetBatchTask(int id)
        {
            return _db.Table<BatchTask>().FirstOrDefault(x => x.Id == id);
        }

        public void UpdateBatchTask(BatchTask batchTask)
        {
            _db.Update(batchTask);
        }

        public Exam[] GetExams(int batchTaskId, int page, DateTime start, DateTime end)
        {
            return _db.Table<Exam>().Where(x => x.BatchTaskId == batchTaskId && x.PageIndex == page && x.CreateTime >= start && x.CreateTime <= end).OrderBy(x => x.Id).ToArray();
        }

        public Image[] GetImages(string examId)
        {
            return _db.Table<Image>().Where(x => x.ExamId == examId).OrderBy(x => x.ImageIndex).ToArray();
        }

        public Exam? GetLastExam(int batchTaskId, DateTime start, DateTime end)
        {
            return _db.Table<Exam>().Where(x => x.BatchTaskId == batchTaskId && x.CreateTime >= start && x.CreateTime <= end).OrderByDescending(x => x.ExamIndex).FirstOrDefault();
        }

        public int GetPageCount(int batchTaskId, string part, string conclusion, DateTime start, DateTime end)
        {
            if (!string.IsNullOrEmpty(part) && !string.IsNullOrEmpty(conclusion))
            {
                var examCount = _db.Table<Exam>().Count(x => x.BatchTaskId == batchTaskId && x.Report.Contains(part) && x.Report.Contains(conclusion) && x.CreateTime >= start && x.CreateTime <= end);
                if(examCount < 15)
                {
                    return 1;
                }
                if(examCount % 15 == 0)
                {
                    return examCount / 15;
                }else
                {
                    return examCount / 15 + 1;
                }              
            }
            else if (!string.IsNullOrEmpty(part))
            {
                var examCount = _db.Table<Exam>().Count(x => x.BatchTaskId == batchTaskId && x.Report.Contains(part) && x.CreateTime >= start && x.CreateTime <= end);
                if (examCount < 15)
                {
                    return 1;
                }
                if (examCount % 15 == 0)
                {
                    return examCount / 15;
                }
                else
                {
                    return examCount / 15 + 1;
                }
            }
            else if (!string.IsNullOrEmpty(conclusion))
            {
                var examCount = _db.Table<Exam>().Count(x => x.BatchTaskId == batchTaskId && x.Report.Contains(conclusion) && x.CreateTime >= start && x.CreateTime <= end);
                if (examCount < 15)
                {
                    return 1;
                }
                if (examCount % 15 == 0)
                {
                    return examCount / 15;
                }
                else
                {
                    return examCount / 15 + 1;
                }
            }
            else
            {
                var lastExam = GetLastExam(batchTaskId, start, end);
                if(lastExam != null)
                {
                    return lastExam.PageIndex + 1;
                }
                return 0;
            }
        }


        public Exam[] GetExams(int batchTaskId, int page, string part, string conclusion, DateTime start, DateTime end)
        {
            if (!string.IsNullOrEmpty(part) && !string.IsNullOrEmpty(conclusion))
            {
                return _db.Table<Exam>().Where(x => x.BatchTaskId == batchTaskId && x.Report.Contains(part) && x.Report.Contains(conclusion) && x.CreateTime >= start && x.CreateTime <= end).OrderBy(x => x.Id).Skip(15 * page).Take(15).ToArray();
            }
            else if (!string.IsNullOrEmpty(part))
            {
                return _db.Table<Exam>().Where(x => x.BatchTaskId == batchTaskId && x.Report.Contains(part) && x.CreateTime >= start && x.CreateTime <= end).OrderBy(x => x.Id).Skip(15 * page).Take(15).ToArray();
            }
            else if (!string.IsNullOrEmpty(conclusion))
            {
                return _db.Table<Exam>().Where(x => x.BatchTaskId == batchTaskId && x.Report.Contains(conclusion) && x.CreateTime >= start && x.CreateTime <= end).OrderBy(x => x.Id).Skip(15 * page).Take(15).ToArray();
            }
            else
            {
                return GetExams(batchTaskId, page, start, end);
            }
        }

        public void Dispose()
        {
            _db?.Dispose();
        }
    }


}
