﻿using PuppeteerSharp;
using System.Text.Encodings.Web;
using System.Text.Json;

namespace StationProbe
{

    internal enum ElementType
    {
        Page,
        Frame
    }

    internal static class ElementConverter
    {
        public static T? As<T>(this object obj) where T : class
        {
            return obj as T;
        }
    }

    internal class ElementCollection
    {
        private readonly Dictionary<ElementType, object> _objects = new();

        public object this[ElementType type] { get { return _objects[type]; } set { _objects[type] = value; } }

        public void AddObject(ElementType element, object obj)
        {
            _objects.Add(element, obj);
        }
    }

    internal interface IExamStore : IDisposable
    {
        Exam? GetLastExam(int batchTaskId);
        void AddExam(Exam exam);
        void AddImage(Image image);
        void UpdateBatchTask(BatchTask batchTask);
    }

    internal abstract class ProbeTask
    {
        public static bool DebugMode { get; set; }

        private static string _debugFolder = string.Empty;

        protected ScreenshotOptions _snapshotOpts = new ScreenshotOptions() { Type = ScreenshotType.Png };

        protected readonly IExamStore _examStore;
        public ProbeTask(IExamStore examStore)
        {
            _examStore = examStore;
        }

        public static void InitDebugFolder()
        {
            if (DebugMode)
            {
                var drives = DriveInfo.GetDrives();
                var largestDriver = drives.OrderByDescending(x => x.TotalFreeSpace).First();
                var debugFolder = largestDriver.Name + "debug";
                if (!Directory.Exists(debugFolder))
                {
                    Directory.CreateDirectory(debugFolder);
                }
                var i = 1;
                while (Directory.Exists(Path.Combine(debugFolder, i.ToString("D6"))))
                {
                    i++;
                }
                _debugFolder = Path.Combine(debugFolder, i.ToString("D6"));
                Directory.CreateDirectory(_debugFolder);
            }
        }

        public void SaveDebugSnapshot(string name, byte[] imageData)
        {
            if (Directory.Exists(_debugFolder))
            {
                var file = Path.Combine(_debugFolder, name);
                File.WriteAllBytes(file, imageData);
            }
        }

        public abstract Task<ElementCollection> ExecuteAsync(ElementCollection inputs);
    }

    internal class LoginTask : ProbeTask
    {
        private readonly string _loginUrl;

        public LoginTask(IExamStore examStore, string loginUrl) : base(examStore)
        {
            _loginUrl = loginUrl;
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            var page = inputs[ElementType.Page].As<IPage>();
            if (page != null)
            {
                await page.GoToAsync(_loginUrl);

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "100000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //await using (var userElement = await page.WaitForSelectorAsync("xpath///input[@id=\"userName\"]"))
                //{
                //    await userElement.FocusAsync();
                //    await page.Keyboard.TypeAsync("justin.xing");
                //}

                //username
                await using (var userElement = await page.WaitForSelectorAsync("input[name=\"username\"]"))
                {
                    await userElement.FocusAsync();
                    await page.Keyboard.TypeAsync("1158");
                }

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "200000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //password
                await using (var passwordElement = await page.WaitForSelectorAsync("input[name=\"userpass\"]"))
                {
                    await passwordElement.FocusAsync();
                    await page.Keyboard.TypeAsync("321321");
                }

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "300000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //login
                await using (var loginButtonElement = await page.WaitForSelectorAsync("img[name=\"inputcode\"]"))
                {
                    await loginButtonElement.ClickAsync();
                    await page.WaitForNetworkIdleAsync();
                }

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "400000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }
            }
            return inputs;
        }
    }

    internal class MenuTask : ProbeTask
    {
        public MenuTask(IExamStore examStore) : base(examStore)
        {
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            Logger.WriteLine("Finding menu...");
            var page = inputs[ElementType.Page].As<IPage>();
            if (page != null)
            {
                await page.GoToAsync("http://192.168.200.85:8080/superimage/front/patientviewQue.jsp");
            }
            return inputs;
        }
    }

    internal class FilterTask : ProbeTask
    {
        private readonly DateTime _start;
        private readonly DateTime _end;

        public int PageCount { get; private set; }

        public int ExamCount { get; private set; }

        public FilterTask(IExamStore examStore, DateTime start, DateTime end) : base(examStore)
        {
            _start = start;
            _end = end;
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            var page = inputs[ElementType.Page].As<IPage>();
            if (page != null)
            {
                await using (var queryLink = await page.WaitForSelectorAsync("span[class=\"link_quedetail\"]"))
                {
                    await queryLink.ClickAsync();
                    await using (var startInputElement = await page.WaitForSelectorAsync("input[name=\"que_startdate\"]"))
                    {
                        await startInputElement.FocusAsync();
                        await page.Keyboard.TypeAsync(_start.ToString("yyyy-MM-dd"));
                    }
                    await using (var endInputElement = await page.WaitForSelectorAsync("input[name=\"que_enddate\"]"))
                    {
                        await endInputElement.FocusAsync();
                        await page.Keyboard.TypeAsync(_end.ToString("yyyy-MM-dd"));
                    }
                    await using (var queryButtonElement = await page.WaitForSelectorAsync("input[name=\"is.query\"]"))
                    {
                        await queryButtonElement.ClickAsync();
                        await Task.Delay(DelayConfig.FilterDelay);
                    }
                }

                await using (var pageTdELement = await page.WaitForSelectorAsync("body > table.tb_page > tbody > tr > td > table > tbody > tr > td:nth-child(1)"))
                {
                    var pageStr = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", pageTdELement);
                    var words = pageStr.Split(' ');
                    ExamCount = int.Parse(words[1].Trim());
                    words = pageStr.Split('/');
                    PageCount = int.Parse(words[1].Trim());
                }

                //await using (var pageinationElement = await page.QuerySelectorAsync("table[class=\"tb_page\"]"))
                //{

                //    await using (var tableElement = await pageinationElement.WaitForSelectorAsync("table"))
                //    {
                //        var tdElements = await tableElement.QuerySelectorAllAsync("td");
                //        if (tdElements.Length == 3)
                //        {
                //            var pageStr = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", tdElements[0]);
                //            var words = pageStr.Split(' ');
                //            ExamCount = int.Parse(words[1].Trim());
                //            words = pageStr.Split('/');
                //            PageCount = int.Parse(words[1].Trim());
                //        }
                //        foreach (var tdElement in tdElements)
                //        {
                //            await tdElement.DisposeAsync();
                //        }
                //    }
                //}
            }
            return inputs;
        }

        internal class PageTask : ProbeTask
        {
            private readonly int _pageIndex;
            public PageTask(IExamStore examStore, int pageIndex) : base(examStore)
            {
                _pageIndex = pageIndex;
            }

            private async Task JumpToPageAsync(IPage page)
            {
                await using (var jumpTableELement = await page.WaitForSelectorAsync("body > table.tb_page > tbody > tr > td > table > tbody > tr > td:nth-child(3) > table"))
                {
                    var jumpTdElements = await jumpTableELement.QuerySelectorAllAsync("td");
                    if (jumpTdElements.Length == 4)
                    {
                        await using (var jumpInputElement = await jumpTdElements[1].WaitForSelectorAsync("input"))
                        {
                            await page.EvaluateFunctionAsync("(element) => element.value=''", jumpInputElement);
                            await jumpInputElement.FocusAsync();
                            await page.Keyboard.TypeAsync((_pageIndex + 1).ToString());
                        }
                        await using (var jumpButtonElement = await jumpTdElements[3].WaitForSelectorAsync("input"))
                        {

                            await jumpButtonElement.ClickAsync();
                            await Task.Delay(DelayConfig.JumpDelay);
                        }
                    }
                    foreach (var jumpTdElement in jumpTdElements)
                    {
                        await jumpTdElement.DisposeAsync();
                    }
                }
            }

            public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
            {
                var page = inputs[ElementType.Page].As<IPage>();
                if (page != null)
                {
                    await JumpToPageAsync(page);
                    if (DebugMode)
                    {
                        var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                        var fileName = "9" + _pageIndex.ToString("D5") + "000.jpg";
                        SaveDebugSnapshot(fileName, imageData);
                    }
                }
                return inputs;
            }
        }

        internal class ExamTask : ProbeTask
        {
            private static readonly ScreenshotOptions _opt = new ScreenshotOptions() { Type = ScreenshotType.Jpeg, Quality = 80 };
            private readonly int _batchTaskId;
            private readonly int _pageIndex;
            private readonly int _examIndex;
            private readonly int _examIndexInPage;

            public ExamTask(IExamStore examStore, int batchTaskId, int pageIndex, int examIndex, int examIndexInPage) : base(examStore)
            {
                _batchTaskId = batchTaskId;
                _pageIndex = pageIndex;
                _examIndex = examIndex;
                _examIndexInPage = examIndexInPage;
            }

            private async Task EnterExamAsync(IPage page)
            {
                await using (var tableElement = await page.WaitForSelectorAsync("table[class=\"tb_main\"]"))
                {
                    var examRecordElements = await tableElement.QuerySelectorAllAsync("tr");
                    var examRecordElement = examRecordElements[_examIndexInPage + 1];
                    var columnElements = await examRecordElement.QuerySelectorAllAsync("td");
                    var nameColumnElement = columnElements[1];
                    await using (var linkElement = await nameColumnElement.WaitForSelectorAsync("a"))
                    {
                        await linkElement.ClickAsync();
                    }
                    foreach (var toDispose in columnElements)
                    {
                        await toDispose.DisposeAsync();
                    }
                    foreach (var toDispose in examRecordElements)
                    {
                        await toDispose.DisposeAsync();
                    }
                }
            }

            private async Task ExitExamAsync(IPage page)
            {
                //Wait for the back button.
                await using (var backButtonElement = await page.WaitForSelectorAsync("input[name=\"Submit222\"]"))
                {
                    await page.EvaluateFunctionAsync("b => b.click()", backButtonElement);
                }
            }

            private async Task DoSaveExamImageAsync(IPage page, string examId, int imageIndex, string imageUrl)
            {
                await using (var imgElement = await page.WaitForSelectorAsync("img"))
                {
                    var imageData = await imgElement.ScreenshotDataAsync(_opt);
                    var image = new Image()
                    {
                        Id = Guid.NewGuid().ToString("N"),
                        ExamId = examId,
                        ImageIndex = imageIndex,
                        Data = imageData,
                        CreateTime = DateTime.Now,
                    };
                    _examStore.AddImage(image);
                    Logger.WriteLine($"Image:{imageUrl} stored. image size:{imageData.Length/1024}kb");
                }
            }

            private async Task SaveExamImageAsync(IPage page, string examId, int imageIndex, string imageUrl)
            {
                await using (var imagePage = await page.Browser.NewPageAsync())
                {
                    await imagePage.GoToAsync(imageUrl);
                    await Task.Delay(DelayConfig.SaveImageDelay);
                    await DoSaveExamImageAsync(imagePage, examId, imageIndex, imageUrl);
                }
            }

            private async Task<List<string>> GetImageUrlsAsync(IPage page)
            {
                //Get all image urls.
                var imageUrls = new List<string>();
                await using (var backButtonElement = await page.WaitForSelectorAsync("input[name=\"Submit222\"]"))
                {
                    await using (var imageTableElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(8)"))
                    {
                        var imageTrs = await imageTableElement.QuerySelectorAllAsync("tr");
                        if (imageTrs != null)
                        {
                            foreach (var imageTr in imageTrs)
                            {
                                var linkElements = await imageTr.QuerySelectorAllAsync("a");
                                foreach (var linkElement in linkElements)
                                {
                                    await using (var imageUrl = await linkElement.GetPropertyAsync("href"))
                                    {
                                        var url = await imageUrl.JsonValueAsync<string>();
                                        imageUrls.Add(url);
                                        await linkElement.DisposeAsync();
                                    }
                                }
                                await imageTr.DisposeAsync();
                            }
                        }
                    }
                }
                Logger.WriteLine($"Get image url count:{imageUrls.Count}");
                return imageUrls;
            }

            private async Task<string> GetReportAsync(IPage page)
            {
                var reportDic = new Dictionary<string, string>();
                await using (var partElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(6) > tbody > tr:nth-child(4) > td:nth-child(2)"))
                {
                    var content = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", partElement);
                    reportDic.Add("Part", content);
                }
                await using (var deviceElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(6) > tbody > tr:nth-child(4) > td:nth-child(4)"))
                {
                    var content = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", deviceElement);
                    reportDic.Add("Device", content);
                }
                await using (var conclusionElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(6) > tbody > tr:nth-child(5) > td:nth-child(4)"))
                {
                    var content = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", conclusionElement);
                    reportDic.Add("Conclusion", content);
                }
                await using (var summaryElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(6) > tbody > tr:nth-child(6) > td.list_cell_bg"))
                {
                    var content = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", summaryElement);
                    reportDic.Add("Summary", content);
                }
                await using (var commentElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(6) > tbody > tr:nth-child(7) > td.list_cell_bg"))
                {
                    var content = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", commentElement);
                    reportDic.Add("Comment", content);
                }
                var report = JsonSerializer.Serialize(reportDic,new JsonSerializerOptions() { WriteIndented = true, Encoder = JavaScriptEncoder.Create(System.Text.Unicode.UnicodeRanges.All)});
                Logger.WriteLine($"Get report:{report}");
                return report;
            }

            public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
            {
                var page = inputs[ElementType.Page].As<IPage>();
                //var frame = inputs[ElementType.Frame].As<IFrame>();
                if (page != null) //&& frame != null)
                {
                    var examId = Guid.NewGuid().ToString("N");
                    //First time enter exam.
                    await EnterExamAsync(page);

                    //wait for page loaded.
                    await Task.Delay(DelayConfig.ExamContentDelay);

                    var imageUrls = await GetImageUrlsAsync(page);
                    for (var i = 0; i < imageUrls.Count; i++)
                    {
                        await SaveExamImageAsync(page, examId, i, imageUrls[i]);
                        //Do not save to fast to avoid network trouble.
                        await Task.Delay(DelayConfig.SaveImageDelay);
                    }

                    if (DebugMode)
                    {
                        var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                        var fileName = "9" + _pageIndex.ToString("D5") + _examIndexInPage.ToString("D3") + ".jpg";
                        SaveDebugSnapshot(fileName, imageData);
                    }


                    //Get patient info
                    var patientName = "";
                    await using (var patientNameElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(2) > tbody > tr:nth-child(1) > td:nth-child(2)"))
                    {
                        patientName = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", patientNameElement);
                    }
                    var patientSex = "";
                    await using (var patientSexElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(2) > tbody > tr:nth-child(1) > td:nth-child(4)"))
                    {
                        patientSex = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", patientSexElement);
                    }

                    var patientAge = 0;
                    await using (var patientAgeElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(2) > tbody > tr:nth-child(2) > td:nth-child(2)"))
                    {
                        var patientAgeStr = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", patientAgeElement);
                        var words = patientAgeStr.Split(' ');
                        if (words.Length == 2)
                        {
                             int.TryParse(words[0].Trim(),out patientAge);
                        }
                    }
                    var examDate = DateTime.Now;
                    await using (var examDateElement = await page.WaitForSelectorAsync("body > form > table:nth-child(1) > tbody > tr:nth-child(2) > td > table > tbody > tr > td:nth-child(2) > table:nth-child(6) > tbody > tr:nth-child(2) > td:nth-child(4)"))
                    {
                        var examDateStr = await page.EvaluateFunctionAsync<string>("(element) => element.textContent", examDateElement);
                        DateTime.TryParse(examDateStr, out examDate);
                    }

                    var report = await GetReportAsync(page);

                    //Save exam.
                    var exam = new Exam()
                    {
                        Id = examId,
                        PatientName = patientName,
                        PatientSex = patientSex,
                        PatientAge = patientAge,
                        ExamDate = examDate,
                        Report = report,
                        BatchTaskId = _batchTaskId,
                        ExamIndex = _examIndex,
                        PageIndex = _pageIndex,
                        ExamIndexInPage = _examIndexInPage,
                        CreateTime = DateTime.Now
                    };
                    _examStore.AddExam(exam);

                    //Back to examl list page.
                    await ExitExamAsync(page);

                    if (DebugMode)
                    {
                        var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                        var fileName = "9" + _pageIndex.ToString("D5") + "999.jpg";
                        SaveDebugSnapshot(fileName, imageData);
                    }
                }
                return inputs;
            }
        }
    }
}
